import { Server } from 'socket.io'
import { TSocket } from '../../utilities/types'
import CharacterRepository from '../../repositories/characterRepository'

type SocketResponseT = {
  character_id: number
}

export default class CharacterConnectEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('character:connect', this.handleCharacterConnect.bind(this))
  }

  private async handleCharacterConnect(data: SocketResponseT): Promise<void> {
    console.log('character:connect requested', data)
    try {
      const character = await CharacterRepository.getByUserAndId(this.socket?.user?.id as number, data.character_id)
      if (!character) return

      this.socket.characterId = character.id
      this.socket.emit('character:connect', character)
    } catch (error: any) {
      console.log('character:connect error', error)
    }
  }
}
