import { Server } from 'socket.io'
import { TSocket } from '../../utilities/types'
import { Character, Zone } from '@prisma/client'
import CharacterRepository from '../../repositories/characterRepository'

type TypePayload = {
  character_id: number
}

type TypeResponse = {
  zone: Zone
  characters: Character[]
}

export default class CharacterDeleteEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('character:delete', this.handleCharacterDelete.bind(this))
  }

  private async handleCharacterDelete(data: TypePayload, callback: (response: TypeResponse) => void): Promise<any> {
    // zod validate
    try {
      await CharacterRepository.deleteByUserIdAndId(this.socket.user?.id as number, data.character_id as number)

      const user_id = this.socket.user?.id as number
      const characters: Character[] = (await CharacterRepository.getByUserId(user_id)) as Character[]

      this.socket.emit('character:list', characters)
    } catch (error: any) {
      console.log(error)
      return this.socket.emit('notification', { message: 'Character delete failed. Please try again.' })
    }
  }
}
