import { Server } from 'socket.io'
import { TSocket } from '../../../../utilities/types'
import { CharacterType } from '@prisma/client'
import characterRepository from '../../../../repositories/characterRepository'
import { gameMasterLogger } from '../../../../utilities/logger'
import CharacterTypeRepository from '../../../../repositories/characterTypeRepository'

interface IPayload {}

export default class CharacterTypeListEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('gm:characterType:list', this.handleCharacterTypeList.bind(this))
  }

  private async handleCharacterTypeList(data: IPayload, callback: (response: CharacterType[]) => void): Promise<void> {
    const character = await characterRepository.getById(this.socket.characterId as number)
    if (!character) {
      gameMasterLogger.error('gm:characterType:list error', 'Character not found')
      return callback([])
    }

    if (character.role !== 'gm') {
      gameMasterLogger.info(`User ${character.id} tried to list character types but is not a game master.`)
      return callback([])
    }

    // get all objects
    const items = await CharacterTypeRepository.getAll()
    callback(items)
  }
}
