import { Server } from 'socket.io'
import { TSocket } from '../../../../utilities/types'
import prisma from '../../../../utilities/prisma'
import characterRepository from '../../../../repositories/characterRepository'

type Payload = {
  id: string
  name: string
  tags: string[]
  originX: number
  originY: number
  isAnimated: boolean
  frameSpeed: number
  frameWidth: number
  frameHeight: number
}

export default class ObjectUpdateEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('gm:object:update', this.handleObjectUpdate.bind(this))
  }

  private async handleObjectUpdate(data: Payload, callback: (success: boolean) => void): Promise<void> {
    const character = await characterRepository.getById(this.socket.characterId as number)
    if (!character) return callback(false)

    if (character.role !== 'gm') {
      return callback(false)
    }

    try {
      const object = await prisma.object.update({
        where: {
          id: data.id
        },
        data: {
          name: data.name,
          tags: data.tags,
          originX: data.originX,
          originY: data.originY,
          isAnimated: data.isAnimated,
          frameSpeed: data.frameSpeed,
          frameWidth: data.frameWidth,
          frameHeight: data.frameHeight
        }
      })
      callback(true)
    } catch (error) {
      console.error(error)
      callback(false)
    }
  }
}
