import { Server } from 'socket.io'
import { TSocket } from '../../../../utilities/types'
import fs from 'fs/promises'
import prisma from '../../../../utilities/prisma'
import characterRepository from '../../../../repositories/characterRepository'
import { getPublicPath } from '../../../../utilities/storage'

export default class SpriteCreateEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('gm:sprite:create', this.handleSpriteCreate.bind(this))
  }

  private async handleSpriteCreate(data: undefined, callback: (response: boolean) => void): Promise<void> {
    try {
      const character = await characterRepository.getById(this.socket.characterId as number)
      if (!character) return callback(false)

      if (character.role !== 'gm') {
        return callback(false)
      }

      const public_folder = getPublicPath('sprites')

      // Ensure the folder exists
      await fs.mkdir(public_folder, { recursive: true })

      const sprite = await prisma.sprite.create({
        data: {
          name: 'New sprite'
        }
      })
      const uuid = sprite.id

      // Create folder with uuid
      const sprite_folder = getPublicPath('sprites', uuid)
      await fs.mkdir(sprite_folder, { recursive: true })

      callback(true)
    } catch (error) {
      console.error('Error creating sprite:', error)
      callback(false)
    }
  }
}
