import { Server } from 'socket.io'
import { TSocket } from '../../../../utilities/types'
import { writeFile } from 'node:fs/promises'
import fs from 'fs/promises'
import prisma from '../../../../utilities/prisma'
import characterRepository from '../../../../repositories/characterRepository'
import { gameMasterLogger } from '../../../../utilities/logger'
import { getPublicPath } from '../../../../utilities/storage'

interface ITileData {
  [key: string]: Buffer
}

export default class TileUploadEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('gm:tile:upload', this.handleTileUpload.bind(this))
  }

  private async handleTileUpload(data: ITileData, callback: (response: boolean) => void): Promise<void> {
    try {
      const character = await characterRepository.getById(this.socket.characterId as number)
      if (!character) return callback(false)

      if (character.role !== 'gm') {
        return
      }

      const public_folder = getPublicPath('tiles')

      // Ensure the folder exists
      await fs.mkdir(public_folder, { recursive: true })

      const uploadPromises = Object.entries(data).map(async ([key, tileData]) => {
        const tile = await prisma.tile.create({
          data: {
            name: 'New tile'
          }
        })
        const uuid = tile.id
        const filename = `${uuid}.png`
        const finalFilePath = getPublicPath('tiles', filename)
        await writeFile(finalFilePath, tileData)
      })

      await Promise.all(uploadPromises)

      callback(true)
    } catch (error) {
      gameMasterLogger.error('Error uploading tile:', error)
      callback(false)
    }
  }
}
